#define minSize 5
#define maxSize 10
#define maxPebbles 100
#define maxIterations 10000

#include <stdio.h>
#include <cstdlib>

const int OK = 0;
const int FAIL = 3;
const int PE = 4;
const int WA = 5;

FILE *fi, *fo, *fa, *fr;
bool report;

void quit(int exitCode, char *message)
{
    if (report)
    {
        fprintf(fr, "%s\n", message);
        fclose(fr);
    }
    else
        printf("%s\n", message);
    fclose(fi);
    fclose(fo);
    fclose(fa);
    exit(exitCode);
}

int a[maxSize + 1][maxSize + 1][maxSize + 1][2 * maxPebbles + 1][4];
int pebbles[maxSize][maxSize], nextPebbles[maxSize][maxSize];

void doTestcase(int size, int A, int B, int X, int Y)
{
    int directions[4][2] = {
        {-1, 0}, // top
        {0, 1},  // right
        {1, 0},  // bottom
        {0, -1}  // left
    };

    for (int i = 0; i < size; i++)
        for (int j = 0; j < size; j++)
            pebbles[i][j] = nextPebbles[i][j] = 0;
    pebbles[0][0] = nextPebbles[0][0] = A;
    pebbles[0][size - 1] = nextPebbles[0][size - 1] = B;

    int iteration = 0;

    while (true)
    {
        iteration++;

        bool movement = false;

        for (int i = 0; i < size; i++)
            for (int j = 0; j < size; j++)
            {
                int move[4] = {0, 0, 0, 0};
                if (pebbles[i][j] > 0)
                    for (int k = 0; k < 4; k++)
                        move[k] = a[size][i + 1][j + 1][pebbles[i][j]][k];
                int moveTotal = 0;
                for (int k = 0; k < 4; k++)
                {
                    if (move[k] < 0)
                        quit(PE, "Negative number of pebbles to move.");
                    else if (move[k] > 0)
                    {
                        int moveI = i + directions[k][0], moveJ = j + directions[k][1];
                        if (moveI < 0 || moveI == size || moveJ < 0 || moveJ == size)
                            quit(PE, "Pebbles to move out of the table.");
                        nextPebbles[moveI][moveJ] += move[k];
                        moveTotal += move[k];
                        movement = true;
                    }
                }
                if (pebbles[i][j] < moveTotal)
                    quit(PE, "Too many pebbles to move.");
                nextPebbles[i][j] -= moveTotal;
            }

        if (! movement || iteration == maxIterations)
        {
            if (! movement)
            {
                if (nextPebbles[size - 1][0] == X && nextPebbles[size - 1][size - 1] == Y)
                    return;
                else
                    quit(WA, "Wrong output.");
            }
            else
                quit(WA, "Iterations limit is reached.");
        }

        for (int i = 0; i < size; i++)
            for (int j = 0; j < size; j++)
                pebbles[i][j] = nextPebbles[i][j];
    }
}

int main(int argc, char *argv[])
{
    if (argc < 4)
    {
        printf("usage: automaton_checker.exe INPUT-FILE CONTESTANT-OUTPUT-FILE AUTHOR-OUTPUT-FILE [REPORT-FILE]\n");
        return 1;
    }
    fi = fopen(argv[1], "r");
    fo = fopen(argv[2], "r");
    fa = fopen(argv[3], "r");
    if (argc >= 5)
    {
        fr = fopen(argv[4], "a");
        report = true;
    } else
        report = false;

    for (int size = minSize; size <= maxSize; size++)
        for (int row = 1; row <= size; row++)
            for (int column = 1; column <= size; column++)
                for (int pebbles = 1; pebbles <= 2 * maxPebbles; pebbles++)
                    for (int i = 0; i < 4; i++)
                        fscanf(fo, "%d", &a[size][row][column][pebbles][i]);

    int subproblem, testcases;
    fscanf(fi, "%d%d", &subproblem, &testcases);

    for (int i = 0; i < testcases; i++)
    {
        int size, A, B, X, Y;
        fscanf(fi, "%d%d%d", &size, &A, &B);
        fscanf(fa, "%d%d", &X, &Y);
        doTestcase(size, A, B, X, Y);
    }

    quit(OK, "OK");
}
